<?php

namespace Kadet\Functional;

use Kadet\Functional\Predicate\AllOfPredicate;
use Kadet\Functional\Predicate\AnyOfPredicate;
use Kadet\Functional\Predicate\ClosurePredicate;
use Kadet\Functional\Predicate\ConstantPredicate;

/**
 * @param $predicate
 *
 * @return \Kadet\Functional\Predicate
 * @throws \Exception
 */
function predicate($predicate): Predicate
{
    if ($predicate instanceof Predicate) {
        return $predicate;
    }

    if (is_callable($predicate)) {
        return new ClosurePredicate(\Closure::fromCallable($predicate));
    }

    // todo better exception
    throw new \Exception();
}

function not($predicate)
{
    return predicate($predicate)->not();
}

function always()
{
    return new ConstantPredicate(true);
}

function never()
{
    return new ConstantPredicate(false);
}

function all(...$predicates)
{
    // extract array
    if (count($predicates) === 1 && is_array($predicates[0])) {
        $predicates = $predicates[0];
    }

    return new AllOfPredicate(...array_map(predicate::class, $predicates));
}

function any(...$predicates)
{
    // extract array
    if (count($predicates) === 1 && is_array($predicates[0])) {
        $predicates = $predicates[0];
    }

    return new AnyOfPredicate(...array_map(predicate::class, $predicates));
}
